/***************************************************************************
 *   Copyright (C) 2007 by Julien Keller   *
 *   j.keller@imperial.ac.uk   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "plotter.h"

//Structures for output, i.e printing, purposes //
enum IntegrationMethod {balanced, einstein, balanced_recursive1,einstein_recursive1}; //enumeration for possible integration methods

struct OutputsStruct //stores the user's choices about the details of outputs, especially which outputs are enabled
{
  OutputsStruct(): printTextA(false), printTextOthers(false), printTextANorm(true), printTextANorm2(false), dispAsCalculate(false), printAsCalculate(false) {} 
  //initialise struct with all outputs off except to print normalized metric 
  ~OutputsStruct() {}
  PlotterOutput dispA;
  PlotterOutput dispANorm;
  PlotterOutput dispANorm2;
  PlotterOutput dispVolumeForms;
  PlotterOutput dispCutOff;
  PlotterOutput dispsv;
  bool dispAsCalculate;
  bool printTextA;
  bool printTextANorm;
  bool printTextANorm2;
  bool printTextOthers;
  bool printAsCalculate;
};


class ResultsStore //class to store useful intemediate results from algorithm
{
  public:
    ResultsStore() {}
    ResultsStore(const ResultsStore & copied): z1(copied.z1), z2(copied.z2), x1(copied.x1), x2(copied.x2), cutOff(copied.cutOff), volumeForm(copied.volumeForm), sv(copied.sv) {}
    ~ResultsStore() {}
            
    const vector<double> & getz1() const {return z1;}
    const vector<double> & getz2() const {return z2;}
    const vector<double> & getx1() const {return x1;}
    const vector<double> & getx2() const {return x2;}
    const vector<double> & getCutOff() const {return cutOff;}
    const vector<double> & getVolumeForm() const {return volumeForm;}
    const vector<double> & getsv() const {return sv;}
            
    void addRow(double z1S, double z2S, double x1S, double x2S, double cutOffS, double volumeFormS, double svS);
    void clear(); //clears all the content of the store
    void print() const; //prints all content in table using cout
      
  protected:
    vector<double> z1;
    vector<double> z2;
    vector<double> x1;
    vector<double> x2;
    vector<double> cutOff;
    vector<double> volumeForm;
    vector<double> sv; //success value, normalised volume form
};


void write_results_in_file(MMatrix<double> *,int);
void print_time(char* a,struct timeval *Tpf,struct timeval *Tps);
        
        