/***************************************************************************
 *   Copyright (C) 2007 by Julien Keller   *
 *   j.keller@imperial.ac.uk   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/





#include "p2blow3ptsfinal.h"



int main(int argc, char **argv)
{
   cout << "ToricHexagon - runs the balancing algorithm for the hexigon" << endl;
   struct timeval *Tpbegin, *Tpend;
   struct timezone *Tzp;
   Tpbegin = (struct timeval*) malloc(sizeof(struct timeval));
   Tpend = (struct timeval*) malloc(sizeof(struct timeval));
   Tzp = 0;
   gettimeofday(Tpbegin, Tzp);
   int k=3; //k is the length of one side of the hexagon, there are k+1 points along one side, so the are (2k+1)^2 coefficients in all. k can be any positive integer.
   OutputsStruct outputs; //a struct storing whether the various forms of output are enabled and details about them
   outputs.dispANorm.enable(); //plotting of normalised metric is on by default
   IntegrationMethod integrMethod = einstein;
   MMatrix<double> former_metric;
   MMatrix<double> * p_former = &former_metric;
   int k_former,kd_former;
   /// reading information from file called param
   ifstream param("param") ;

   if(!param.is_open())
   {
      cout << "No input parameter file found \n Using standard values" << endl;
   }
   else
   {
      cout << "Reading input parameters from 'param' file";
      param >> k ;
      skipline(param);
      param >> method;
      if(method==0)
      {
         integrMethod = balanced;
         cout << "\n Balanced method ";
      }
      if(method==1)
      {
         integrMethod = einstein;
         cout << "\n Ricci flow method ";
      }
      if(method==2)
      {
         integrMethod = balanced_recursive1;
         cout << "\n Balanced and recursive (1 step) method";
      }
      if(method==3)
      {
        integrMethod = einstein_recursive1;
        cout << "\n Ricci and recursive (1 step) method";
      }

      skipline(param);
      param >> integrSteps;
      skipline(param);
      param >> numIterations;
      skipline(param);
      param >> choice_output;
      skipline(param);
      param >> zoom;
      skipline(param);
      param >> write_result;
      skipline(param);
      param >> gchar;
      skipline(param);
      param >> print_results;
      skipline(param);
      param >> sleep_time;
      skipline(param);
      param >> random_init;
      skipline(param);
      if(method==2 || method==3)
      {
         param >> former_metric; //get metric from previous computation
         show(former_metric,"\n Previous metric in memory");
      }
      param.close();

   }

   //INITIALISATION
   int kd = 2 * k + 1; //kd is the number of points along one side of the box containing the hexagon. there are kd^2 coefficients.

   //blank matrices to represent the coefficients at grid points in the polygon's box - see assignments following
   MMatrix<double> matrix1(kd,kd);
   MMatrix<double> matrix2(kd,kd);
   MMatrix<double> matrix3(kd,kd);
   MMatrix<double> matrix4(kd,kd);

   MMatrix<double> * p_ar; //Pointer to ar, the previous but one result of the algorithm
   MMatrix<double> * p_a = &matrix1; //Pointer to a, the current matrix to be processed
   MMatrix<double> * p_an = &matrix2; //Pointer to an, the new matrix that is the result of the algorithm
   MMatrix<double> * p_anNorm = &matrix3; //Pointer to anNorm, a normalised version of an
   MMatrix<double> * p_anNorm2 = &matrix4; //Pointer to anNorm2, a version of an normalised further

   //initialise the initial matrix (anNorm as this is immediately moved into a) to 1 for all points inside the polygon
   srand(time(NULL));
   for(int p=0; p < kd; p++)
   {
      for(int q=0; q < kd; q++)
      {
        if(random_init==1){
          if(inPolygon(p, q, k)) (*p_anNorm)[p][q] = ((rand() % 1000)*PI)/(rand() % 1000); 
          else (*p_anNorm)[p][q] = 0;
        }
        if(random_init==0){
          if(inPolygon(p, q, k)) (*p_anNorm)[p][q] = 1; 
          else (*p_anNorm)[p][q] = 0;
        }
       }
   }

   ResultsStore additResults; //a struct to store any useful values worked out by the algorithm other than the new set of coefficients

   int iterationLoop = 0;
   bool quitFlag = false;

  plotDisp(outputs, p_anNorm, p_anNorm, p_anNorm2, additResults, iterationLoop); //initial display command for any plotter windows up by default - NB p_anNorm is given in place of p_an as anNorm contains the inital values that really an should contain, but doesn't to save work


   //ALGORITHM LOOP
   handle=make_plotter(choice_output,zoom,handle);
   if (method==0)
   {
      printTextonPlotter("Balanced method",0.5 ,0.9);
   }
   if (method==1)
   {
      printTextonPlotter("Ricci flow method",0.5 ,0.9);
   }
   if (method==2)
   {
      printTextonPlotter("Balanced method with 1 step recursive",0.5 ,0.9);
   }
   if (method==3)
   {
     printTextonPlotter("Ricci flow method with 1 step recursive",0.5 ,0.9);
   }

   while (iterationLoop < numIterations)
   {

      if(gchar==1)
      {
         cout << "\n Press key to continue.\n";
         getchar();
      }
      //advance the pointers to the matricies rather than copying matricies
      p_ar = p_a;
      p_a = p_anNorm; //normalised version from last time is what this iteration of algorithm is performed on
      p_anNorm = p_an;
      

      //perform the algorithm
      if (iterationLoop>0) pl_erase();
      printTitle("Scalar curvature");
      printIteration(iterationLoop+1);

      if(iterationLoop>1)
      {
         printScalarcurvature(Mresults[0][2],Mresults[0][3]);
      }
      Mresults.fillWith(1);
      Mresults[0][2]=-1;

      p_an = doAlgorithm(p_a, p_ar,p_former, additResults, integrSteps, integrMethod); //the new matrix is overwritten over the old p_ar
   
      normaliseResults(p_an, p_anNorm);
      normaliseResults2(p_anNorm, p_anNorm2);

      if (outputs.dispAsCalculate || iterationLoop + 1 == numIterations)
      {
         plotDisp(outputs, p_an, p_anNorm, p_anNorm2, additResults, iterationLoop + 1);
      }
      if (outputs.printAsCalculate || iterationLoop + 1 == numIterations)
      {
         printText(outputs, p_an, p_anNorm, p_anNorm2, additResults, iterationLoop + 1);
      }

      cout << "\n Completed iteration " << iterationLoop + 1 << endl;
      iterationLoop++;
   }


   if(print_results==1){
     cout<<"\n Max and Min of scalar curvature : "<< Mresults[0][2] << " "<< Mresults[0][3] <<endl;
   }
   if(write_result==1)
   {
      write_results_in_file(p_an,method);
   }
   show(Mresults,"1st: number of points, 2nd: average scalar, 3rd & 4th : max and min of scalar curvature\n");
   gettimeofday(Tpend, Tzp);
   print_time("\n-> Time for finding points on the manifold ",Tpend,Tpbegin);
   cout << "\n ---------------------------------------\n Press key to terminate.";
   getchar();

   return EXIT_SUCCESS;
}



MMatrix<double> * doAlgorithm(const MMatrix<double> * p_a, MMatrix<double> * p_an, MMatrix<double> *p_former, ResultsStore & additResults, int integrSteps, IntegrationMethod integrMethod)
{
   //Easy references to the matrix being processed and target matrix


   const MMatrix<double> & a = *p_a;
   MMatrix<double> & an = *p_an;
   MMatrix<double> & an_former=*p_former;
   int kd_former=an_former.getHeight();
   int k_former=(int) (kd_former-1)/2;
   int kd = a.getHeight();
   int k = kd / 2;
   pl_space (-1, -1,1, 1); /* defines coordinates of the system */
   int prev=0;
   double prev_x,prev_y;
   additResults.clear();

   int z1Loop;
   int z2Loop;
   int z2LoopLast = 0;
   double cutOff;
   double volumeForm;
   MMatrix<double> vm; //3x3 matrix whose determinant is volume form
   MMatrix<double> vm_former; //3x3 matrix whose determinant is volume form for previous metric
   vm.fillWith(0.5);
   MMatrix<double> ip (kd, kd);
   ip.fillWith(0);
   double integrStepWidth = HORIZ_Z_INTEGR_OVER / ((double) integrSteps); //width of integration steps
   double scalar,scalar_former;// contains curvature results

   for(z1Loop = 0; z1Loop < integrSteps; z1Loop++)
   {
      z2Loop = 0;
      cutOff = 1.;
      while( (cutOff > 0 || z2Loop + integrSteps/6. <= z2LoopLast) && z2Loop < (integrSteps * Z2LOOP_CAP_FACTOR)) //loop until reach the cut off except when reached very soon to avoid short sections on high z1Loop where low z2Loops give cutOff 0 before increasing again - also cap to prevent infinite loop if cut off not reached due to error //
      {
         
         double z1 = integrStepWidth * z1Loop; //z is the variable integrated over, z=e^t
         double z2 = integrStepWidth * z2Loop;
         
                 
         vm = findVolumeFormMatrix(a, z1, z2);//not optimized
         
         if(method!=1)
         {
         volumeForm = determinant3x3(vm) / pow3(vm[0][0]); //only useful in balanced method = should be an option to save time 
         }
         //x gives the coordinates of the point within the hexagon - relative to axes with origin in bottom left corner of hexigon box and (1,1) in the top right
         double x1 = z1 * vm[0][1] / (2. * k * vm[0][0]);
         double x2 = z2 * vm[0][2] / (2. * k * vm[0][0]);
         
         cutOff = cutOffFunct(ellipseFunct(x1,x2)) / rotationFunct(x1,x2);
         
         if (cutOff != 0 && !isnan(cutOff) && finite(cutOff))
         {

            compute_curvatures_local(p_an,x1,x2,scalar);
            Mresults[0][0]++;
            Mresults[0][1]=((Mresults[0][0]-1)*Mresults[0][1]+scalar)/Mresults[0][0];
            Mresults[0][2]=max(Mresults[0][2],scalar);
            Mresults[0][3]=min(Mresults[0][3],scalar);
            if(method==2 ||method==3)
            {
               compute_curvatures_local(p_former,x1,x2,scalar_former);
            }
            switch(integrMethod)
            {
            case balanced:
               addIntegrationStep(ip, z1, z2, pow2(integrStepWidth) * volumeForm * cutOff / vm[0][0]);
               break;
            case einstein:
               addIntegrationStep(ip, z1, z2, pow2(integrStepWidth) *  cutOff / pow(vm[0][0], 1. + 1. / k));
               break;
            case balanced_recursive1:
               vm_former = findVolumeFormMatrix(an_former, z1, z2);
               //cout<<"\n Bergman former metric "<<vm_former[0][0]/(kd_former)<< "| " <<                   (1./kd)*(1-pow2(k_former)-k_former*scalar_former/2.)<< "  "<<               (1./kd)*(vm[0][0]/vm_former[0][0]-pow2(k_former)-k_former)<< "|" << (((double) k_former)/kd)*(1-scalar_former);
               addIntegrationStep(ip, z1, z2, pow2(integrStepWidth) * cutOff * volumeForm* (1 +(((double) k_former)/kd)*(1-scalar_former))/vm[0][0]);
              break; 
              case  einstein_recursive1:
                addIntegrationStep(ip, z1, z2, pow2(integrStepWidth) * cutOff *  (1 +(((double) k_former)/kd)*(1-scalar_former))/pow(vm[0][0], 1. + 1. / k) );
               break;
            default:
               throw("Error! Invalid integration method variable value.");
            }
         }

         double sv =(method!=2 && method!=3)? volumeForm *  pow(vm[0][0], 1./((double)k)):1; //a measure of success - a normalisation of the volume form
         //store useful intemediate results for display
         additResults.addRow(z1, z2, x1, x2, cutOff, volumeForm, sv);

         //loop for output //
         if(prev==0)
         {
            prev_x=0;
            prev_y=0;
            prev=1;
            
         }

         if (!(  abs(prev_x -x1+0.5 )<0.005 && abs(prev_y - x2+0.5)<0.005 && isnan(scalar)))
         {
            color_grade2(scalar);
           // color_grade3(scalar);
            pl_fpoint(x1-0.5, x2-0.5);
            double nx1 = x1-0.5;
            double nx2 = x2-0.5;
            double nnx2;
            int it2;
            for(it2=0;it2<5;it2++)
            {

               nnx2 = -nx1 + nx2;
               nx1 = nx2; ///nx2
               nx2 = nnx2;
               if(cutOff!=0)
               {
                  pl_fpoint(nx1,nx2);
               }
            }
            prev_x=x1;
            prev_y=x2;
         }


         z2Loop++;
      }
      z2LoopLast = z2Loop;
   }

   //copy and rotate the integrals just calculated around the 6 corners of the hexigon to calculate the full integrals
   an.fillWith(0);
   for(int p=0; p < kd; p++)
   {
      for(int q=0; q < kd; q++)
      {
         if (inPolygon(p,q,k))
         {//TODO Could precalculate the repeated sums, reorder sums for clarify?
            an[p][q] = ip[p][q] + ip[p-q+k][p] + ip[kd-q-1][p-q+k] + ip[kd-p-1][kd-q-1] + ip[q-p+k][kd-p-1] + ip[q][q-p+k];
         }
      }
   }
   invertNonZeroElements(an);
   sleep(sleep_time);
   return p_an;
}

void addIntegrationStep(MMatrix<double> & ip, const double z1, const double z2, const double factor)
{
   int kd = ip.getHeight();
   int k = kd / 2;

   double z1_raisedto_2pplus1 = z1;
   for(int p=0; p < kd; p++)
   {
      //assert(ARE_EQUAL_FLOATS( z1_raisedto_2pplus1, pow(z1, (double)2*p+1) ));

      double z2_raisedto_2qplus1 = z2;
      for(int q=0; q < kd; q++)
      {
       //  assert(ARE_EQUAL_FLOATS( z2_raisedto_2qplus1, pow(z2, (double)2*q+1) ));
         if (inPolygon(p,q,k)) //TODO is this if causing more work than it saves?
         {
            ip[p][q] += factor * z1_raisedto_2pplus1 * z2_raisedto_2qplus1;
         }
         z2_raisedto_2qplus1 *= pow2(z2);
      }

      z1_raisedto_2pplus1 *= pow2(z1);
   }
}

MMatrix<double> findVolumeFormMatrix(const MMatrix<double> & a, double z1, double z2)
{
   int kd = a.getHeight();
   int k = kd / 2;
   MMatrix<double> vm(3,3); //volume form matrix
   vm.fillWith(0);

   //work out the similar sums that are all a different factor away from the required sums for elements of vm, not including cases when p or q is 0. These cases cause complications by giving 2p-2 or 2q-2 values less than 0, as z1 or z2 might equal 0. Also when p or q equal zero some terms of the sum are 0 and may be excluded.
   double z1_raisedto_2pminus2 = 1;
   double z2_raisedto_2qminus2;
   for(int p=1; p < kd; p++)
   {
       //assert(ARE_EQUAL_FLOATS( z1_raisedto_2pminus2, pow(z1, (double)2*p-2) ));
      double sum = 0.;
      double sumq = 0.;
      double sumq2 = 0.;

      z2_raisedto_2qminus2 = 1.;
      for(int q=1; q < kd; q++)
      {
        // assert(ARE_EQUAL_FLOATS( z2_raisedto_2qminus2, pow(z2, (double)2*q-2) ));
         double delpq = a[p][q] * z1_raisedto_2pminus2 * z2_raisedto_2qminus2;
         sum += delpq;
         sumq += delpq * q;
         sumq2 += delpq * pow2(q);

         z2_raisedto_2qminus2 *= pow2(z2);
      }

      vm[0][0] += sum;
      vm[0][1] += sum * p;
      vm[0][2] += sumq;
      vm[1][1] += sum * pow2(p);
      vm[1][2] += sumq * p;
      vm[2][2] += sumq2;

      z1_raisedto_2pminus2 *= pow2(z1);
   }

   vm[0][0] *= pow2(z1) * pow2(z2);
   vm[0][1] *= z1 * pow2(z2);
   vm[0][2] *= pow2(z1) * z2;
   vm[1][1] *= pow2(z2);
   vm[1][2] *= z1 * z2;
   vm[2][2] *= pow2(z1);

   z1_raisedto_2pminus2 = 1;
   for(int p=1; p < kd; p++)
   {
      //assert(ARE_EQUAL_FLOATS( z1_raisedto_2pminus2, pow(z1, (double)2*p-2) ));

      vm[0][0] += a[p][0] * z1_raisedto_2pminus2 * pow2(z1); //z2 is raised to power 1 for these 3 terms
      vm[0][1] += a[p][0] * p * z1_raisedto_2pminus2 * z1;
      vm[1][1] += a[p][0] * pow2(p) * z1_raisedto_2pminus2;
      //the terms for the other elements are 0 as they are multiplied by q

      z1_raisedto_2pminus2 *= pow2(z1);
   }
   //when p==0 and q!=0
   z2_raisedto_2qminus2 = 1;
   for(int q=1; q < kd; q++)
   {
      //assert(ARE_EQUAL_FLOATS( z2_raisedto_2qminus2, pow(z2, (double)2*q-2) ));

      vm[0][0] += a[0][q] * z2_raisedto_2qminus2 * pow2(z2); //z1 is raised to power 1 for these 3 terms
      vm[0][2] += a[0][q] * q * z2_raisedto_2qminus2 * z2;
      vm[2][2] += a[0][q] * pow2(q) * z2_raisedto_2qminus2;
      //the terms for the other elements are 0 as they are multiplied by p
      z2_raisedto_2qminus2 *= pow2(z2);
   }
   //when q==0 and p==0
   vm[0][0] += a[0][0]; //z1 and z2 are raised to power 1 for this term

   //fill the other elements of vm by its symmetry
   vm[1][0] = vm[0][1];
   vm[2][0] = vm[0][2];
   vm[2][1] = vm[1][2];
   return vm;
}

void normaliseResults(const MMatrix<double> * p_an, MMatrix<double> * p_anNorm) //TODO - Improve this normalisation to give smaller numbers with large k (that is k > 20 approx)
{

   const MMatrix<double> & an = *p_an;
   MMatrix<double> & anNorm = *p_anNorm;
   int kd = an.getHeight();
   int k = kd / 2;

   anNorm =  pow(3., 2. * ((double)k) ) * an / an.sum();
}

void normaliseResults2(const MMatrix<double> * p_anNorm, MMatrix<double> * p_anNorm2)
{
   const MMatrix<double> & anNorm = *p_anNorm;
   MMatrix<double> & anNorm2 = *p_anNorm2;
   int kd = anNorm.getHeight();
   int k = kd / 2;

   for(int p=0; p < kd; p++)
   {
      for(int q=0; q < kd; q++)
      {
         if (inPolygon(p, q, k))
         {
            anNorm2[p][q] = -1. / ((double)k) * log(anNorm[p][q]);
         }
      }
   }
}

bool inPolygon(int p, int q, int k)
{
   return std::abs(p - q) <= k;
}

double cutOffFunct(double y)
{
   double c;
   double step=0.1;
   if (y <= step)
   {
      c = 1;
   }
   else
   {
      if (y >= 1.+step)
      {
         c = 0;
      }
      else
      { //that is when 0.3 < y < 1.2
         c = 0.5 * (cos(PI * (y - step)) + 1.);
      }
   }
   return c;
}

double ellipseFunct(double x1, double x2)
{
   return pow2(8./3.) * (pow2(((x1 + x2) / 2. - .25))) + pow2(1./.3) * (pow2((x1 - x2)));
}

double rotationFunct(double x1, double x2)
{
   double rot = 0.;
   double nx1;
   double nx2;
   for (int i = 0; i < 6; i++)
   {
      rot += cutOffFunct(ellipseFunct(x1,x2));
      nx1 = x2;
      nx2 = x2 - x1 + 0.5;

      x1 = nx1;
      x2 = nx2;
   }
   return rot;
}



/////////////////////////////////////
//Plotting Function Implementations//
/////////////////////////////////////
void plotDisp(OutputsStruct & outputs, const MMatrix<double> * p_an, const MMatrix<double> * p_anNorm, const MMatrix<double> * p_anNorm2, const ResultsStore & additResults, int iterationsComplete)
{
   //Easy references to the matrix being shown

   const MMatrix<double> & an = *p_an;
   const MMatrix<double> & anNorm = *p_anNorm;
   const MMatrix<double> & anNorm2 = *p_anNorm2;

   if (outputs.dispA.isEnabled())
   {
      outputs.dispA.selectPlotter();
      pl_erase();
      drawHexigon();
      printTitle("Unnormalised Metric");
      printIteration(iterationsComplete);

      plotHexMatrix(an, red);
   }

   if (outputs.dispANorm.isEnabled())
   {
      outputs.dispANorm.selectPlotter();
      pl_erase();
      drawHexigon();
      printTitle("Normalised Metric");
      printIteration(iterationsComplete);

      plotHexMatrix(anNorm, redBlue);
   }

   if (outputs.dispANorm2.isEnabled())
   {
      outputs.dispANorm2.selectPlotter();
      pl_erase();
      drawHexigon();
      printTitle("Further Normalised Metric");
      printIteration(iterationsComplete);

      plotHexMatrix(anNorm2, blue);
   }

   if (outputs.dispCutOff.isEnabled())
   {
      outputs.dispCutOff.selectPlotter();
      pl_erase();
      drawHexigon();
      pl_pencolorname ("pink");
      pl_fellipse(0.25, 0.25, 0.6, 3./8.*sqrt(2.4), -45.); //TODO - fix these numbers for cutoff ellipse
      printTitle("Cut Off Values");
      printIteration(iterationsComplete);

      plotHexPoints(additResults.getx1(), additResults.getx2(), additResults.getCutOff(), blueRedHard0);
   }

   if (outputs.dispVolumeForms.isEnabled())
   {
      outputs.dispVolumeForms.selectPlotter();
      pl_erase();
      drawHexigon();
      printTitle("Volume Form");
      printIteration(iterationsComplete);

      plotHexPoints(additResults.getx1(), additResults.getx2(), additResults.getVolumeForm(), blue);
   }

   if (outputs.dispsv.isEnabled())
   {
      outputs.dispsv.selectPlotter();
      pl_erase();
      drawHexigon();
      printTitle("Normalised Volume Form");
      printIteration(iterationsComplete);

      plotHexPoints(additResults.getx1(), additResults.getx2(), additResults.getsv(), redBlue);
   }


}

void printText(OutputsStruct & outputs, const MMatrix<double> * p_an, const MMatrix<double> * p_anNorm, const MMatrix<double> * p_anNorm2, const ResultsStore & additResults, int iterationsComplete)
{
   //Easy references to the matrix being shown

   const MMatrix<double> & an = *p_an;
   const MMatrix<double> & anNorm = *p_anNorm;
   const MMatrix<double> & anNorm2 = *p_anNorm2;

   if (outputs.printTextA)
   {
      show(an, "metric");
   }

   if (outputs.printTextANorm)
   {
      show(an, "normalised metric");
   }

   if (outputs.printTextANorm2)
   {
      show(an, "further normalised metric");
   }

   if (print_results==2 ||outputs.printTextOthers)
   {
      additResults.print();
   }

}

void drawHexigon()
{
   pl_pencolorname ("green");
   pl_fline(0.,0.,0.,0.5);
   pl_fline(0.,0.,0.5,0.);
   pl_fline(0.,0.5,0.5,1.);
   pl_fline(0.5,0.,1.,0.5);
   pl_fline(1.,0.5,1.,1.);
   pl_fline(0.5,1.,1.,1.);
}



void plotHexMatrix(const MMatrix<double> & M, ColourGradient gradient)
{



   int kd = M.getHeight();
   int k = kd / 2;
   double min = findHexMatrixMin(M);
   double max = findHexMatrixMax(M);
   double mean = findHexMatrixMean(M);

   //plot the points for the matrix in graded colours
   for(int p=0; p < kd; p++)
   {
      for(int q=0; q < kd; q++)
      {
         if (inPolygon(p, q, k))
         {
            Colour colour = colourGrade(M[p][q], min, max, gradient);
            pl_pencolor(colour.red, colour.green, colour.blue);

            double x1 = ((double) p) / ((double) (kd-1));
            double x2 = ((double) q) / ((double) (kd-1));
            pl_fmarker(x1, x2, PLOTTER_MARKER_TYPE_MATRIX, PLOTTER_MARKER_SIZE_POINTS);
         }
      }
   }

   //print the maximum, minimum and mean values of the entries of the Matrice that we get finally to give a sense of scale
   
   pl_fontname ("Helvetica");
   pl_fontsize(-1);
   pl_pencolorname ("black");

   pl_fmove(0.90, 0.07);
   std::stringstream smMax;
   smMax << " Max: " << max;
   pl_label(smMax.str().c_str());

   pl_fmove(0.90, 0.05);
   std::stringstream smMin;
   smMin  << " Min: " << min;
   pl_label(smMin.str().c_str());

   pl_fmove(0.90, 0.03);
   std::stringstream smMean;
   smMean  << " Mean: " << mean;
   pl_label(smMean.str().c_str());
   
}



void plotHexPoints(const vector<double> & x1, const vector<double> & x2, const vector<double> & v, ColourGradient gradient)
{


   double min = vectorMin(v);
   double max = vectorMax(v);
   double mean = vectorMean(v);

   //plot the points for the matrix in graded colours
   for(int i=0; i < x1.size(); i++)
   {
      Colour colour = colourGrade(v[i], min, max, gradient);
      pl_pencolor(colour.red, colour.green, colour.blue);

      pl_fmarker(x1[i], x2[i], PLOTTER_MARKER_TYPE_POINTS, PLOTTER_MARKER_SIZE_POINTS);
   }

   //print the maximum, minimum and mean values to give a sense of scale
   pl_fontname ("Helvetica");
   pl_fontsize(-1);
   pl_pencolorname ("black");

   pl_fmove(0.90, 0.07);
   std::stringstream smMax;
   smMax << " Max: " << max;
   pl_label(smMax.str().c_str());

   pl_fmove(0.90, 0.05);
   std::stringstream smMin;
   smMin  << " Min: " << min;
   pl_label(smMin.str().c_str());

   pl_fmove(0.90, 0.03);
   std::stringstream smMean;
   smMean  << " Mean: " << mean;
   pl_label(smMean.str().c_str());

}

double findHexMatrixMax(const MMatrix<double> & M)
{

   int kd = M.getHeight();
   int k = kd / 2;

   double v = M[0][0];
   for(int p=0; p < kd; p++)
   {
      for(int q=0; q < kd; q++)
      {
         if (inPolygon(p, q, k))
         {
            if (M[p][q] > v) v = M[p][q];
         }
      }
   }

   return v;
}

double findHexMatrixMin(const MMatrix<double> & M)
{

   int kd = M.getHeight();
   int k = kd / 2;

   double v = M[0][0];
   for(int p=0; p < kd; p++)
   {
      for(int q=0; q < kd; q++)
      {
         if (inPolygon(p, q, k))
         {
            if (M[p][q] < v) v = M[p][q];
         }
      }
   }
   return v;
}

double findHexMatrixMean(const MMatrix<double> & M)
{
   int kd = M.getHeight();
   int k = kd / 2;

   double total = 0;
   double elements = 0;
   for(int p=0; p < kd; p++)
   {
      for(int q=0; q < kd; q++)
      {
         if (inPolygon(p, q, k))
         {
            elements++;
            total += M[p][q];
         }
      }
   }

   return total / elements;
}

////////////////////////////////////
//Utility Function Implementations//
////////////////////////////////////

double vectorMax(vector<double> v)
{
   double m = v[0];
   for(int i=1; i < v.size(); i++)
   {
      if (v[i] > m)
      {
         m = v[i];
      }
   }
   return m;
}

double vectorMin(vector<double> v)
{
   double m = v[0];
   for(int i=1; i < v.size(); i++)
   {
      if (v[i] < m)
      {
         m = v[i];
      }
   }
   return m;
}

double vectorMean(vector<double> v)
{
   double s = 0;
   for(int i=0; i < v.size(); i++)
   {
      s += v[i];
   }
   return s / ((double) v.size());
}


void skipline(ifstream & mystream)
{
   mystream.ignore(100,'\n');
}
//////////////////////////////
//Computation of Curvatures //
//////////////////////////////

void compute_curvatures_local(MMatrix<double> * p_an,double z1,double z2,double& scalar )
{

   const MMatrix<double> & an = *p_an;

   int kd = an.getHeight();
   int k=kd-1;
   double factor=3./2.*k/6.;//This is 27/18=Area/Number boundary points=1/ mean scalar curvature
   MMatrix<double> A(k+6,k+6);

   A.fillWith(0,0,an,0,0);
   MMatrix<double> D(k+6,5);
   MMatrix<double> DD(5,5);
   double G,F,p10,p01,p20,p11,p02,p30,p21,p12,p03;
   double p40,p04,p31,p13,p22;
   double hxx,hyy,hxy,det,ixx,ixy,iyy,cPx,cRy,cQx,cQy,cRx,cPy;
   double phiPP,phiRR,phiPQ,phiQR,phiPR,phiQQ;
   double BPP,BRR,BPQ,BQR,BPR,BQQ;

   double ricxx,ricyy,ricxy,scal,detricci,tfricsize,gauss,beta,gamma;
   double W[5];
   double inv[2][2];
   double S[2][2][2][2];



   double w1=z1*z1;
   double w2=z2*z2;
   D.fillWith(0);
   for (int i=0;i<=k+5;i++)
   {
      for (int r=0; r<=k; r++)
      {
    
         D[i][0]=D[i][0]*w2+A[i][k-r];
         if(k-r+1<=kd)
         {
            D[i][1]=D[i][1]*w2+A[i][k-r+1]*(k-r+1);
         }
         if(k-r+2<=kd)
         {
            D[i][2]=D[i][2]*w2+A[i][k-r+2]*(k-r+2)*(k-r+1);
         }
         if(k-r+3<=kd)
         {
            D[i][3]=D[i][3]*w2+A[i][k-r+3]*(k-r+3)*(k-r+2)*(k-r+1);
         }
         if(k-r+4<=kd)
         {
            D[i][4]=D[i][4]*w2+A[i][k-r+4]*(k-r+4)*(k-r+3)*(k-r+2)*(k-r+1);
         }
      }

   }

   //show(D,"D");
   DD.fillWith(0);
   for (int a=0;a<=4;a++)
   {
      for (int r=0;r<=k;r++)
      {
         DD[0][a]=DD[0][a]*w1+D[k-r][a];
         DD[1][a]=DD[1][a]*w1+D[k-r+1][a]*(k-r+1);
         DD[2][a]=DD[2][a]*w1+D[k-r+2][a]*(k-r+2)*(k-r+1);
         DD[3][a]=DD[3][a]*w1+D[k-r+3][a]*(k-r+3)*(k-r+2)*(k-r+1);
         DD[4][a]=DD[4][a]*w1+D[k-r+4][a]*(k-r+4)*(k-r+3)*(k-r+2)*(k-r+1);
      }
   }



   G=(1.+w1+w2);

   F=DD[0][0];
   p10=DD[1][0]/F;//pab denotes derivatives of phi=logF,a derivs in w1 and b in w2;
   p01=DD[0][1]/F;
   p20=DD[2][0]/F-p10*p10;
   p11=DD[1][1]/F-p10*p01;
   p02=DD[0][2]/F-p01*p01;
   p30=3.*p10*p20+p10*p10*p10;
   p21=2.*p10*p11+p01*p20+p10*p10*p01;
   p12=2.*p01*p11+p10*p02+p01*p01*p10;
   p03=3.*p01*p02+p01*p01*p01;
   p30=(DD[3][0]/F)-p30;
   p21=(DD[2][1]/F)-p21;
   p12=(DD[1][2]/F)-p12;
   p03=(DD[0][3]/F)-p03;
   p40=4.*p10*p30+3.*p20*p20+6.*p20*p10*p10+p10*p10*p10*p10;
   p04=4.*p01*p03+3.*p02*p02+6.*p02*p01*p01+p01*p01*p01*p01;
   p31=3.*p10*p21+p01*p30+3.*p11*p20+3.*p20*p10*p01+3.*p11*p10*p10+p01*p10*p10*p10;
   p13=3.*p01*p12+p10*p03+3.*p11*p02+3.*p02*p01*p10+3.*p11*p01*p01+p10*p01*p01*p01;
   p22=2.*p10*p12+2.*p01*p21+p20*p02+2.*p11*p11+p20*p01*p01+p02*p10*p10;
   p22=p22+4.*p11*p10*p01+p10*p10*p01*p01;
   p40=(DD[4][0]/F)-p40;
   p31=(DD[3][1]/F)-p31;
   p22=(DD[2][2]/F)-p22;
   p13=(DD[1][3]/F)-p13;
   p04=(DD[0][4]/F)-p04;



   hxx=p10+w1*p20;//h is the metric tensor; x and y are other names for  z1,z2
   hyy=p01+w2*p02;
   hxy= z1*z2*p11;
   det= hxx*hyy-hxy*hxy;

   ixx=hyy/det; //i is the inverse (upper indices)
   ixy=-hxy/det;
   iyy=hxx/det;

   cPx= z1*(2.*p20+ w1*p30);//these are the third deriv. terms
   cRy= z2*(2.*p02+ w2*p03);
   cQx= z2*(p11+w1*p21);
   cQy= z1*(p11+w2*p12);
   cRx= w2*z1*p12;
   cPy= w1*z2*p21;

   phiPP=2.*p20+4.*w1*p30+w1*w1*p40;//these are the fourth deriv. terms
   phiRR=2.*p02+4.*w2*p03+w2*w2*p04;
   phiPQ=z2*z1*(2.*p21+ w1*p31);
   phiQR=z1*z2*(2.*p12+w2*p13);
   phiPR=w1*w2*p22;
   phiQQ= p11+w1*p21+w2*p12+w1*w2*p22;

   BPP=cPx*cPx*ixx+(cPy*cPx+cPx*cPy)*ixy+cPy*cPy*iyy;//this computes quadratic in third derivs.
   BRR=cRx*cRx*ixx+(cRy*cRx+cRx*cRy)*ixy+cRy*cRy*iyy;
   BPQ=cPx*cQx*ixx+(cPx*cQy+cQx*cPy)*ixy+cPy*cQy*iyy;
   BQR=cQx*cRx*ixx+(cQx*cRy+cRx*cQy)*ixy+cQy*cRy*iyy;
   BPR=cPx*cRx*ixx+(cPx*cRy+cRx*cPy)*ixy+cPy*cRy*iyy;
   BQQ=cQx*cQx*ixx+(cQx*cQy+cQy*cQx)*ixy+cQy*cQy*iyy;

   BPP=-factor*(phiPP-BPP);
   BRR=-factor*(phiRR-BRR);
   BPQ=-factor*(phiPQ-BPQ);
   BQR=-factor*(phiQR-BQR);
   BPR=-factor*(phiPR-BPR);
   BQQ=-factor*(phiQQ-BQQ);//now BPQ etc are components of curvature tensor, "factor"normalises scale

   ricxx= (BPP*ixx+2.*BPQ*ixy+BQQ*iyy);//contract for Ricci
   ricyy=(BRR*iyy+2.*BQR*ixy+BQQ*ixx);
   ricxy=(BPQ*ixx+BPR*ixy+BQQ*ixy+BQR*iyy);

   scal=ricxx*ixx+2.*ricxy*ixy+ricyy*iyy;
   
   
   detricci=ricxx*ricyy-ricxy*ricxy;
   tfricsize= sqrt(scal*scal-4.*detricci/det);//absolute value of Ricci eigenvalue difference
   gauss=(BPR-BQQ)/(4.*det);// Gauss curvature in horizontal plane, up to a factor.
   beta=gauss-scal/8.;
   gamma=2.*det*beta;

   W[0]=BPP-ricxx*hxx-beta*hxx*hxx;
   W[1]=BPQ-.5*(ricxx*hxy+ricxy*hxx)-beta*hxx*hxy;
   W[2]=BPR-.5*(ricxx*hyy+ricyy*hxx)-beta*hxx*hyy-gamma;
   W[3]=BQR-.5*(ricyy*hxy+ricxy*hyy)-beta*hyy*hxy;
   W[4]=BRR-ricyy*hyy-beta*hyy*hyy;


   inv[0][0]=ixx;
   inv[0][1]=ixy;
   inv[1][0]=ixy;
   inv[1][1]=iyy;



   for (int i=0;i<=1;i++)
   {
      for (int j=0;j<=1;j++)
      {
         for (int k=0;k<=1;k++)
         {
            for (int l=0;l<=1;l++)
            {
               S[i][j][k][l]=W[i+j+k+l];
            }
         }
      }
   }

   double modcurv=0.;

   for (int i=0;i<=1;i++)
   {
      for (int j=0;j<=1;j++)
      {
         for(int k=0;k<=1;k++)
         {
            for(int l=0;l<=1;l++)
            {
               for(int a=0;a<=1;a++)
               {
                  for(int b=0;b<=1;b++)
                  {
                     for(int c=0;c<=1;c++)
                     {
                        for(int d=0;d<=1;d++)
                        {
                           modcurv=modcurv+S[i][j][k][l]*S[a][b][c][d]*inv[i][a]*inv[j][b]*inv[k][c]*inv[l][d];
                        }
                     }
                  }
               }
            }
         }
      }
   }
   modcurv=sqrt(modcurv);
   if (print_results==1){
     cout<<"\n curvatures (scalar,det of Ricci,Gauss) "<<scal<< " "<<detricci<< " "<<gauss;
   }
   scalar=scal;
}






