/***************************************************************************
 *   Copyright (C) 2007 by Julien Keller   *
 *   j.keller@imperial.ac.uk   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "plotter.h"


/////////////////////////////////
//PlotterOutput Implementations//
/////////////////////////////////


/******************************************************************************/

int make_plotter(int choice_output,int zoom,int& handle)
{

//  int handle; /* handle to name the plotter */
  char *display_type;
  char *bitmap_size;

  if (choice_output==0)
  {
    display_type = "X";/* format of plotter */
  }
  if (choice_output==1)
  {
    display_type = "png";
  }
  if(zoom==2)
  {
    bitmap_size="600x600"; /* size in pixels of plotter */
  }
  if(zoom==3)
  {
    bitmap_size="860x860"; /* size in pixels of plotter */
  }
  if(zoom==1)
  {
    bitmap_size="450x450"; /* size in pixels of plotter */
  }
  handle=init_plotsession(display_type,bitmap_size);
  return(handle);
}

int init_plotsession(char *display_type, char *bitmap_size)
{
  int handle, return_value;
  return_value=pl_parampl("BITMAPSIZE", bitmap_size);
  if(return_value<0)
  {
    fprintf(stderr,"The plotter does not accept the parameters specified.\n");
    exit(1);
  }
  handle=pl_newpl(display_type,stdin, stdout, stderr);

  if(handle<0)
  {
    fprintf(stderr,"Can't create plotter.\n");
    exit(1);
  }
  return_value=pl_selectpl(handle);
  if(return_value<0)
  {
    fprintf(stderr,"Plotter does not exist\n");
    exit(1);
  }
  return_value=pl_openpl();
  if(return_value<0)
  {
    fprintf(stderr,"Plotter can't be opened.\n");
    exit(1);
  }
  return handle;
}



int end_plotsession(int handle) /* closes the plotter -> 0 if OK */  {
   int err_status=0, return_value;
   return_value= pl_closepl(); /* ferme la session du plotter */
   if(return_value<0)
   {
     fprintf(stderr,"Problem to close the plotter\n");
     /* no exit, because we try to delete the plotter */
     err_status +=1;
   }
   return_value=pl_selectpl(0);
   if(return_value<0)
   {
     fprintf(stderr,"Default Plotter could not be selected!\n");
     err_status +=1<<1;
   }
   /* deleting our plotter */
   return_value=pl_deletepl (handle);
   if(return_value<0)
   {
     fprintf(stderr,"Cannot delete plotter !\n");
     err_status +=1<<2;
   }
   return err_status;
}

/*
void color_grade2(double f,int& prev_color)
{
  if (f < 0.5 && prev_color!=1)
  {
    pl_pencolorname("black");  //1
    prev_color=1;

  }
  if(f > 0.5 && f < 0.6 && prev_color!=2)
  {
    pl_pencolorname("violet");
    prev_color=2;
  }
  if(f > 0.6 && f< 0.8  && prev_color!=3)
  {
    pl_pencolorname("blue");
    prev_color=3;
  }
  if(f > 0.8 && f < 1  && prev_color!=4)
  {
    pl_pencolorname("green");
    prev_color=4;
  }
  if(f > 1 &&f < 1.2 && prev_color!=5)
  {
    pl_pencolorname("greenyellow");
    prev_color=5;
  }

  if(f > 1.2 && f < 1.4 && prev_color!=6)
  {
    pl_pencolorname("yellow");
    prev_color=6;
  }
  if(f > 1.4 && f < 1.6 && prev_color!=7)
  {
    pl_pencolorname("orange");
    prev_color=7;
  }
  if(f > 1.5 && prev_color!=8)
  {
    pl_pencolorname("red");
    prev_color=8;
  }
}
*/

void color_grade2(double f)
{
    if (f < 0.65 )
    {
        pl_pencolorname("black");  //1
    }
    if (f > 0.65 && f < 0.75)
    {
        pl_pencolorname("darkblue");  //1
    }
    
    if(f > 0.75 && f < 0.85 )
    {
        pl_pencolorname("darkcyan");     
    }
    
    if(f > 0.85 && f< 0.95 )
    {
        pl_pencolorname("cyan2");
      
    }
    if(f > 0.95 && f < 0.98 )
    {
        pl_pencolorname("green4");
        
    }
    if(f > 0.98 && f < 1.02 )
    {
        pl_pencolorname("green");
    }
    if(f > 1.02 && f < 1.05 )
    {
        pl_pencolorname("greenyellow");
    }
    if(f > 1.05 &&f < 1.15)
    {
        pl_pencolorname("yellowgreen");
        
    }
    if(f > 1.15 && f < 1.25)
    {
        pl_pencolorname("orange");
        
    }
    if(f > 1.25 && f<1.35)
    {
        pl_pencolorname("orange4");
        
    }
    if(f > 1.35 ) 
    {
        pl_pencolorname("red");
        
    }
}

void color_grade3(double f)
{
  Colour c;
  c.red=0;c.blue=0;c.green=0;
  double range = 1;
  double proportUpRange;
  if (f<0.50){
    proportUpRange=0;
  }
  if(f>0.5+range){
    proportUpRange=range;
  }
  else{
    proportUpRange=f-0.5;
  }
  proportUpRange=proportUpRange/range;
  if(proportUpRange>0.75){
     c.red = (int) ( COLOUR_MAX * proportUpRange );
  }
  if( 0.25< proportUpRange && proportUpRange  <0.50){
    
    c.green = (int) ( COLOUR_MAX * 2*proportUpRange ); 
    c.blue = (int) ( COLOUR_MAX * (1-proportUpRange ));   
  }
  if( 0.50< proportUpRange && proportUpRange  <0.75){
    
    c.green = (int) ( COLOUR_MAX * 2*(1-proportUpRange) ); 
    c.red = (int) ( COLOUR_MAX * proportUpRange );
  }
  if( proportUpRange <0.25){
    c.blue = (int) ( COLOUR_MAX * (1-proportUpRange ));
  }
  pl_pencolor(c.red,c.green,c.blue);
}


Colour colourGrade(double value, double min, double max, ColourGradient gradient) //TODO this is not a very efficient function - could use function pointers to multiple colourGradient Functions rather than switch statement.
    // Not used
{

  Colour c;
  c.red = 0;
  c.green = 0;
  c.blue = 0;
  double range = max - min;
  double proportUpRange;
  if (range == 0)
  {
    proportUpRange = 1;
  }
  else
  {
    proportUpRange = (value - min)/range;
  }

  switch(gradient) //TODO - define more gradients
  {
    case red:
    {
      c.red = (int) ( COLOUR_MAX * proportUpRange );
    }
    break;
    case blue:
    {
      c.blue = (int) ( COLOUR_MAX * proportUpRange );
    }
    break;
    case green:
    {
      c.green = (int) ( COLOUR_MAX * proportUpRange );
    }
    break;
    case 3:
    {
      c.red = (int) ( COLOUR_MAX * (1. - proportUpRange) );
      c.blue = (int) ( COLOUR_MAX * proportUpRange );
    }
    break;
    case blueRedHard0:
    {
      if (value <= 0)
      {
        c.red = (int) ( COLOUR_MAX );
      }
      else
      {
        c.blue = (int) ( COLOUR_MAX );
      }
    }
    break;
    default:
      throw("Invalid Colour Gradient value!");
  }

  return c;
}






void PlotterOutput::selectPlotter()
{
  int return_value = pl_selectpl(plotter);
  if (return_value < 0)
  {
    throw("Could not select plotter!\n");
  }
}



void PlotterOutput::toggleOnOff()
{
  if (enabled)
  {
    disable();
  }
  else
  {
    enable();
  }
}

void PlotterOutput::enable()
{
    char *bitmap_size2;
    bitmap_size2="400x400"; /* size in pixels of plotter */
    pl_parampl("BITMAPSIZE", bitmap_size2);

  if (!enabled)
  {
    
    plotter=pl_newpl("X",stdin, stdout, stderr);
    if(plotter < 0)
    {
      throw("Can't create plotter.\n");
    }

    int return_value=pl_selectpl(plotter);
    if(return_value < 0)
    {
      throw("Newly created plotter cannot be selected\n");
    }

    return_value=pl_openpl();
    if(return_value < 0)
    {
      throw("Plotter can't be opened.\n");
    }

    enabled = true;

    pl_fspace (PLOTTER_WINDOW_COORDINATES);

    pl_erase();
  }

}

void PlotterOutput::disable()
{
  if (enabled)
  {
    int return_value;

      //if the currently selected plotter is the one to be deleted, select the default plotter (0)
    int current_selection = pl_selectpl(0);
    if (current_selection != plotter)
    {
      return_value = pl_selectpl(current_selection);
      if (return_value < 0)
      {
        throw("Could not return to last selected plotter!\n");
      }
    }
    else if (current_selection < 0)
    {
      throw("Default Plotter could not be selected!\n");
    }

      // delete this output's plotter
    return_value = pl_deletepl (plotter);
    if(return_value < 0)
    {
      throw("Cannot delete plotter !\n");
    }

    plotter = -1;
    enabled = false;
  }
}

/****************/



void printScalarcurvature(double scalarmax,double scalarmin)
{
  std::stringstream streamMessage;
  pl_fontname ("Helvetica");
  pl_fontsize(-1);
  pl_pencolorname ("black");
  if (scalarmax==-1){
    streamMessage  << "Max and Min of scalar curvature (previous iteration): inf " <<scalarmin;
  }
  
  else{
    streamMessage  << "Max and min of Scalar curvature (previous iteration): "<< scalarmax << " "<<scalarmin;
  }
  pl_fmove(-0.75, -0.75);
  pl_label(streamMessage.str().c_str());
}


void printIteration(int iterationsComplete)
{
   //Print number of iterations so far on plotter window
  std::stringstream streamMessage;
  pl_fontname ("Helvetica");
  pl_fontsize(-1);
  pl_pencolorname ("black");
  pl_fmove(0.05, 0.85);
  streamMessage  << "Iteration: " << iterationsComplete;
  pl_label(streamMessage.str().c_str());
}

void printTitle(const string & title)
{
   //Print title on current plotter window
  pl_fontname ("Helvetica");
  pl_fontsize(-1);
  pl_pencolorname ("black");
  pl_fmove(0.05, 0.95);
  pl_label(title.c_str());
}



void printTextonPlotter(const string & title,float x1,float x2)
{
  pl_fontname ("Helvetica");
  pl_fontsize(-1);
  pl_pencolorname ("black");
  pl_fmove(x1,x2);
  pl_label(title.c_str());
}
