/***************************************************************************
 *   Copyright (C) 2007 by Julien Keller   *
 *   j.keller@imperial.ac.uk   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef __PLOTTER_H__ //inclusion guard
#define __PLOTTER_H__

#include <fstream>
#include "m_matrix2.h"  //some procedures for standard operations on matrices
#include "plotter.h"  //some procedures for handling plotters 
#include <stdio.h> //standard basic library
#include <stdlib.h> //standard basic library
#include <iostream> //standard library for text input and output
#include <string>//standard library for easy to use strings
#include <sstream> //standard library to make combining strings easier
#include <vector> //standard library for the std::vector class
#include <plot.h>
//Constants for Tweaking the Program
#define PLOTTER_WINDOW_COORDINATES -0.01, -0.01, 1.1, 1.1 // Coordinates of the top left and bottom right corners of displayed plotter windows, defining the plotter coordiate system. Set to allow a 1x1 box to fit. The sides of the hexigon are 0.5 units long on this scale. Origin is to the bottom left hand corner of hexigon
#define PLOTTER_MARKER_TYPE_MATRIX 16 //set the type of the points drawn by the plotters when plotting matricies (eg the metric) (2 is a small cross)
#define PLOTTER_MARKER_SIZE_MATRIX 0.01 //set the size of the points drawn by the plotters when plotting matricies (eg the metric) (in the units of the plotter)
#define PLOTTER_MARKER_TYPE_POINTS 2 //set the type of the points drawn by the plotters when plotting collections of points (eg the volume forms) (2 is a small cross)
#define PLOTTER_MARKER_SIZE_POINTS 0.05 //set the size of the points drawn by the plotters when plotting collections of points (eg the volume forms) (in the units of the plotter)

//Constants from libraries
#define COLOUR_MAX 65535 //maximum colour intensity value in the plot library for the colour values for red, green and blue



class PlotterOutput //manages an X window plotter and whether or not to plot to it.
{
  public:
    PlotterOutput(): enabled(false), plotter (-1) {} //initially the output is off
    PlotterOutput(const PlotterOutput & copied) {enabled = copied.enabled; plotter = copied.plotter;} //copy constructor - copies shallowly and does not create new plotter
    ~PlotterOutput() {}
            
    bool isEnabled() const {return enabled;}
            
    void enable(); //enable this output and open the window for the output - leaves this output's plotter selected
    void disable();
    void toggleOnOff();

    void selectPlotter(); //select this output's plotter for plotting to
            
  protected:
    bool enabled;
    int plotter;
}; 
 
 

struct Colour //an RGB colour value
{
  int red;
  int green;
  int blue;
};

enum ColourGradient {red, green, blue, redBlue, blueRedHard0}; //possibly colour gradients for plots

//simple procedures to handle with plotters //
int make_plotter(int,int,int&);
int init_plotsession(char *, char *);
int end_plotsession(int handle);
void setPlotterResol(const string & bitmap_size); //set the resolution of the plotter windows (has to be the same for all of them)

//Color grade results //
void color_grade2(double); //simple color grade function
void color_grade3(double); //simple color grade function
Colour colourGrade(double value, double min, double max, ColourGradient gradient); //gives the appropriate RGB colour value in a colour gradient for value given the values assigned to the top and bottom of the gradient. It must be that max >= min. If max = min then the high colour of the gradient is returned.

//Printing results// 
void printIteration(int iterationsComplete); //prints a label in the corner of the currently selected plotter window with the number of interations so far
void printTitle(const string & title); //Print title on currently selected plotter window
void printScalarcurvature(double ,double ); //print the scalar curvature (min and max) values of the previous iteration
void printTextonPlotter(const string & ,float ,float ); //print a message at coord x1,x2
#endif
